{-# LANGUAGE Strict, TypeFamilies #-}

module Api.V1.Types (
  Request (..)
  , Response (..)
  , ResponseFor
  , GetSections (GetSections)
  , Section (..)
  , GetTopics (GetTopics)
  , TopicInfo (..)
  , GetTopic (..)
  , Topic (..)
  , ThreadHead (..)
  , ThreadPost (..)
  , threadHeadToPost
) where

import Relude

import Data.Aeson (FromJSON (parseJSON), ToJSON (toJSON), Value (Object), withObject, (.:))

data Request
  = IGetSections GetSections
  | IGetTopics GetTopics
  | IGetTopic GetTopic
  deriving stock (Eq, Show)

instance Hashable Request where
  hashWithSalt s (IGetSections r) = hashWithSalt s r
  hashWithSalt s (IGetTopics r) = hashWithSalt s r
  hashWithSalt s (IGetTopic r) = hashWithSalt s r

instance ToJSON Request where
  toJSON (IGetSections r) = toJSON r
  toJSON (IGetTopics r) = toJSON r
  toJSON (IGetTopic r) = toJSON r

data Response
  = OGetSections (ResponseFor GetSections)
  | OGetTopics (ResponseFor GetTopics)
  | OGetTopic (ResponseFor GetTopic)
  deriving stock (Generic, Show)

instance NFData Response

instance FromJSON Response where
  parseJSON v =
    OGetSections <$> parseJSON v
      <|> OGetTopics <$> parseJSON v
      <|> OGetTopic <$> parseJSON v

type family ResponseFor a where
  ResponseFor GetSections = [Section]
  ResponseFor GetTopics = [TopicInfo]
  ResponseFor GetTopic = Topic

data GetSections = GetSections
  deriving stock (Eq, Show)

instance Hashable GetSections where
  hashWithSalt = const

instance ToJSON GetSections where
  toJSON GetSections = Object $ fromList [("type", "sections")]

data Section = Section
  { id :: Integer
  , thread_group_name, description :: Text
  }
  deriving stock (Generic, Show)

instance NFData Section

instance FromJSON Section

data GetTopics = GetTopics
  deriving stock (Eq, Show)

instance Hashable GetTopics where
  hashWithSalt = const

instance ToJSON GetTopics where
  toJSON GetTopics = Object $ fromList [("type", "threads")]

data TopicInfo = TopicInfo
  { id, group_id, nbr_ansfers :: Integer
  , thrname, date, name, last_activity_date, last_activity_user :: Text
  }
  deriving stock (Generic, Show)

instance NFData TopicInfo

instance FromJSON TopicInfo

newtype GetTopic = GetTopic {topic_id :: Integer}
  deriving stock (Eq, Show)
  deriving newtype (Hashable)

instance ToJSON GetTopic where
  toJSON GetTopic{topic_id} =
    Object $
      fromList
        [ ("type", "topic")
        , ("key", "anon")
        , ("id", show topic_id)
        ]

data Topic = Topic
  { thread :: ThreadHead
  , post :: [ThreadPost]
  }
  deriving stock (Generic, Show)

instance NFData Topic

instance FromJSON Topic

data ThreadHead = ThreadHead
  { info :: TopicInfo
  , message :: Text
  }
  deriving stock (Generic, Show)

instance NFData ThreadHead

instance FromJSON ThreadHead where
  parseJSON = withObject "ThreadHead" \o ->
    ThreadHead <$> parseJSON (Object o)
      <*> o .: "message"

threadHeadToPost :: ThreadHead -> ThreadPost
threadHeadToPost ThreadHead{info, message} =
  ThreadPost
    { id = info.id
    , topic_id = info.id
    , name = info.name
    , date = info.date
    , message
    }

data ThreadPost = ThreadPost
  { id, topic_id :: Integer
  , name, date, message :: Text
  }
  deriving stock (Generic, Show)

instance NFData ThreadPost

instance FromJSON ThreadPost
