{-# OPTIONS_GHC -Wno-orphans #-}

module Servant.API.ContentTypes.AtomFeed.Instances () where

import Relude

import Api.V1.Types
import Feed.V1.Types (TopicNew (..), TopicsNew (..))
import Servant.API.ContentTypes.AtomFeed (IsAtomFeed (..))
import Text.Atom.Feed
import Utils (renderCommonmarkViaHtml)

instance IsAtomFeed Topic where
  toAtomFeed Topic{thread, post} =
    feed
      { feedIcon = Just "http://4rum.i2p/favicon.ico"
      , feedEntries
      }
   where
    posts = threadHeadToPost thread : post
    feed =
      nullFeed
        ("http://4rum.i2p/topic/" <> show thread.info.id)
        (TextString $ "4rum - " <> thread.info.thrname)
        thread.info.last_activity_date
    feedEntries = postToEntry <$> posts
    postToEntry ThreadPost{id = post_id, name, date, message} =
      let entry =
            nullEntry
              ("http://4rum.i2p/topic/" <> show thread.info.id <> "#" <> show post_id)
              (TextString name)
              date
       in entry
            { entryContent = Just . HTMLContent . renderCommonmarkViaHtml $ message
            }

instance IsAtomFeed TopicsNew where
  toAtomFeed TopicsNew{section_id, topics} =
    feed
      { feedIcon = Just "http://4rum.i2p/favicon.ico"
      , feedEntries
      }
   where
    feed =
      nullFeed
        ("http://4rum.i2p/sections/" <> show section_id)
        (TextString $ "4rum - " <> "Section #" <> show section_id)
        (foldr max "0000-00-00 00:00:00" ((.date) <$> topics))
    feedEntries = topicToEntry <$> topics
    topicToEntry TopicNew{id = topic_id, thrname, date, name, message} =
      let entry =
            nullEntry
              ("http://4rum.i2p/topic/" <> show topic_id)
              (TextString $ thrname <> " by " <> name)
              date
       in entry
            { entryContent = Just . HTMLContent . renderCommonmarkViaHtml $ message
            }
