{-# LANGUAGE Strict #-}
{-# OPTIONS_GHC -Wno-orphans #-}

module Api.Color (
  GetSections (GetSections)
  , Section (..)
  , GetTopics (GetTopics)
  , TopicInfo (..)
  , GetTopic (..)
  , Topic (..)
  , ThreadHead (..)
  , ThreadPost (..)
) where

import Relude

import Api.V1.Types
import Color.Print
import Color.Utils
import Polysemy.Commonmark.AST (commonmark)
import Polysemy.Output.IO
import Polysemy.Sgr
import System.Console.ANSI

instance PrintColor Section where
  printColor Section{id = thread_id, thread_group_name, description} = do
    withSgr [SetUnderlining SingleUnderline] do
      putTextO thread_group_name
    withSgr [SetColor Foreground Dull White] do
      putTextLnO $ " #" <> show thread_id
    putTextLnO description

instance PrintColor TopicInfo where
  printColor TopicInfo{id = topic_id, thrname, last_activity_date, last_activity_user} = do
    withSgr [SetUnderlining SingleUnderline] do
      putTextO thrname
    putTextO ": "
    withSgr [SetPaletteColor Foreground (textColor last_activity_user)] do
      putTextO $ "@" <> last_activity_user
    withSgr [SetColor Foreground Dull White] do
      putTextLnO $ " at " <> last_activity_date <> " #" <> show topic_id

instance PrintColor Topic where
  printColor Topic{thread, post} =
    sequenceA_ $ intersperse (putTextLnO "") (printColor <$> threadHeadToPost thread : post)

instance PrintColor ThreadPost where
  printColor ThreadPost{id = post_id, name, date, message} = do
    withSgr
      [ SetPaletteColor Foreground (textColor name)
      , SetConsoleIntensity BoldIntensity
      -- , SetSwapForegroundBackground True
      ]
      do
        putTextO $ "@" <> name
    withSgr [SetColor Foreground Dull White] do
      putTextLnO $ " " <> date <> " #" <> show post_id
    case commonmark "<message>" message of
      Right block -> printColor block
      Left err -> withSgr [SetColor Foreground Vivid Red] do
        putTextLnO $ "Error parsing commonmark: " <> show err
        putTextLnO message
